/* Tracing routines for KISS TNC
 * Copyright 1991 Phil Karn, KA9Q
 *
 * Modified by G1EMM  19/11/90 to support multiport KISS mode.
 *
 * ANSICOLOR Mods by KA1NNN 12/94
 */
/* Tracing to sockets, mods April '95 by Johan. K. Reinalda, WG7J */
  
#include "global.h"
#include "mbuf.h"
#include "kiss.h"
#include "devparam.h"
#include "ax25.h"
#include "trace.h"
#include "socket.h"
  
extern int LocalTraceColor;    /* KA1NNN 12/94 - search 'LocalTraceColor' */

void
ki_dump(s,bpp,check)
int s;
struct mbuf **bpp;
int check;
{
    int type,t;
    int val;

    if (LocalTraceColor == 2)
        usprintf(s,"\33[1;37m");
    if (LocalTraceColor == 1)
        usprintf(s,"\33[0;36m");       
#ifdef MONITOR
    if (!Trace_compact_header)
#endif
        usprintf(s,"KISS: ");
    type = PULLCHAR(bpp);
    t = type & 0x0f;
    if(t == PARAM_DATA || t == PARAM_CRCREQ){
#ifdef MONITOR
        if (!Trace_compact_header)
#endif
            usprintf(s,"Port %d Data\n", type >> 4);
        if (LocalTraceColor)
            usprintf(s,"\33[0;37m");
        ax25_dump(s,bpp,check);
        return;
    }
    if(type == PARAM_RETURN){
        usprintf(s,"RETURN\n");
        if (LocalTraceColor)
            usprintf(s,"\33[0;37m");
        return;
    } else {
#ifdef MONITOR
        if (!Trace_compact_header)
#endif
            usprintf(s,"Port %d ", type >> 4);
    }
    val = PULLCHAR(bpp);
    switch(t){
        case PARAM_TXDELAY:
            usprintf(s,"TX Delay: %lu ms\n",val * 10L);
            if (LocalTraceColor)
                usprintf(s,"\33[0;37m");
            break;
        case PARAM_PERSIST:
            usprintf(s,"Persistence: %u/256\n",val + 1);
            if (LocalTraceColor)
                usprintf(s,"\33[0;37m");
            break;
        case PARAM_SLOTTIME:
            usprintf(s,"Slot time: %lu ms\n",val * 10L);
            if (LocalTraceColor)
                usprintf(s,"\33[0;37m");
            break;
        case PARAM_TXTAIL:
            usprintf(s,"TX Tail time: %lu ms\n",val * 10L);
            if (LocalTraceColor)
                usprintf(s,"\33[0;37m");
            break;
        case PARAM_FULLDUP:
            usprintf(s,"Duplex: %s\n",val == 0 ? "Half" : "Full");
            if (LocalTraceColor)
                usprintf(s,"\33[0;37m");
            break;
        case PARAM_HW:
            usprintf(s,"Hardware %u\n",val);
            if (LocalTraceColor)
                usprintf(s,"\33[0;37m");
            break;
        case PARAM_POLL:
            usprintf(s,"Poll\n");
            if (LocalTraceColor)
                usprintf(s,"\33[0;37m");
            break;
        default:
            usprintf(s,"code %u arg %u\n",type,val);
            if (LocalTraceColor)
                usprintf(s,"\33[0;37m");
            break;
    }
}
  
int
ki_forus(iface,bp)
struct iface *iface;
struct mbuf *bp;
{
    struct mbuf *bpp;
    int i;
  
    if((bp->data[0] & 0x0f) != PARAM_DATA)
        return 0;
    dup_p(&bpp,bp,1,AXALEN);
    i = ax_forus(iface,bpp);
    free_p(bpp);
    return i;
}
